"""
cli.pkg.clone - Vhex package cloning user interface
"""
import sys

import core.pkg
from core.logger import log

__all__ = [
    'clone_pkg_cli'
]

#---
# Internals
#---

__HELP__ = """
vxsdk-pkg-clone
Package manager cloning part

USAGE:
    vxsdk pkg clone [ OPTIONS ] [ targets[, ...]]

DESCRIPTION:
    The installation part is simple, you juste need to refer the repository
    name and, optionally, the version information. Like this:

        <NAME@version> (ex: vxKernel@1.2.3)

    This will allow the searching to try to match package with specific version
    information. This part is very powerfull because If the 'version' is
    detected to respect the correct semantic versioning, you can perform
    version operations in the version target:

        Caret requirements (^)
            ^1.2.3  :=  >=1.2.3, <2.0.0
            ^1.2    :=  >=1.2.0, <2.0.0
            ^1      :=  >=1.0.0, <2.0.0
            ^0.2.3  :=  >=0.2.3, <0.3.0
            ^0.2    :=  >=0.2.0, <0.3.0
            ^0.0.3  :=  >=0.0.3, <0.0.4
            ^0.0    :=  >=0.0.0, <0.1.0
            ^0      :=  >=0.0.0, <1.0.0

        Tilde requirements (~)
            ~1.2.3  := >=1.2.3, <1.3.0
            ~1.2    := >=1.2.0, <1.3.0
            ~1      := >=1.0.0, <2.0.0

        Wildcard requirements (*)
            *       := >=0.0.0
            1.*     := >=1.0.0, <2.0.0
            1.2.*   := >=1.2.0, <1.3.0

    Note that it's possible that a package can have two same version (one for
    branch name and another for a tag), by default, the tag is always selected,
    but here the display information will explicitly describe if the version is
    a tag and / or a branch.

OPTIONS:
    -y, --yes               Do not ask for interactive confirmation
    -c, --confirm           Ask for interactive confirmation
    -o, --output=<path>     Clone (or link) at <path>
        --bare              Do not clone the package in the global storage
    -h, --help              Print this help and exit
""".strip()

#---
# Public
#---

def clone_pkg_cli(argv):
    """ Clone a particular package """
    if not argv:
        log.error(__HELP__)
        sys.exit(84)
    if '-h' in argv or '--help' in argv:
        log.user(__HELP__)
        sys.exit(0)

    output = ''
    bare = False
    confirm = True
    workaround = False
    package_target = ''
    for arg in argv:
        if workaround:
            output = arg
            workaround = False
            continue
        if arg == '-o':
            workaround = True
            continue
        if arg in ['-y', '--yes', '-c', '--confirm']:
            confirm = arg in ['-c', '--confirm']
            continue
        if arg.find('--output=') == 0:
            output = arg[9:]
            continue
        if arg == '--bare':
            bare = True
            continue
        if not package_target:
            package_target = arg.split('@')
            if len(package_target) < 2:
                package_target = [arg, '']
            continue
        log.error(f"argument '{arg}' unrecognized")
        sys.exit(84)

    path = core.pkg.clone(
        package_target[0],
        None if len(package_target) != 2 else package_target[1],
        output,
        confirm,
        bare
    )
    return 0 if path else -1
