"""
core.build.cmake - CMake abstraction
"""
import os
import sys
import subprocess

__all__ = [
    'cmake_configure',
    'cmake_build',
    'cmake_install',
    'cmake_uninstall',
]

#---
# Public
#---

def cmake_configure(pkg_meta, _, env):
    """ Abstract cmake configuration """
    prefix_src   = f"{pkg_meta.path}"
    prefix_build = f"{pkg_meta.build_prefix}/build/{pkg_meta.name}"
    toolchain_flag = ''
    if toolchain_path := env.get('VXSDK_HOOK_CMAKE_TOOLCHAIN'):
        toolchain_flag = f"-DCMAKE_TOOLCHAIN_FILE={toolchain_path}"
    shell_cmd    = f"cmake {toolchain_flag} -B {prefix_build} -S {prefix_src}"
    return subprocess.run(shell_cmd.split(), check=False).returncode

def cmake_build(pkg_meta, verbose, _):
    """ Abstract cmake configuration """
    prefix_build = f"{pkg_meta.build_prefix}/build/{pkg_meta.name}"
    shell_cmd    = f"cmake --build {prefix_build}"
    if verbose:
        shell_cmd += ' --verbose'
    return subprocess.run(shell_cmd.split(), check=False).returncode

def cmake_install(pkg_meta, verbose, _):
    """ Abstract cmake installation """
    prefix_build = f"{pkg_meta.build_prefix}/build/{pkg_meta.name}"
    shell_cmd    = f"cmake --install {prefix_build}"
    if verbose:
        shell_cmd += ' --verbose'
    return subprocess.run(shell_cmd.split(), check=False).returncode

def cmake_uninstall(pkg_meta, verbose, _):
    """ Abstract cmake uninstall

    Note that CMake does not offert a easy way to uninstall project, but it
    generate a file which contains all installed pathname
    """
    prefix_build = f"{pkg_meta.build_prefix}/build/{pkg_meta.name}"
    manifile     = f"{prefix_build}/install_manifest.txt"
    if not os.path.exists(manifile):
        print('project not installed')
        return -1

    retcode = 0
    with open(manifile, 'r', encoding='utf8') as manifest:
        for pathname in manifest.readlines():
            pathname = pathname.strip()
            if not os.path.exists(pathname):
                continue
            if verbose:
                print("-- Removing {pathname}")
            ret = subprocess.run(f"rm {pathname}".split(), check=False)
            if ret.returncode == 0:
                continue
            print("warning : error during removing file", file=sys.stderr)
            retcode -= 1
    return retcode
