"""
core.build.env - Handle env configuration
"""
import os

from core.logger import log
from core.config import config

__all__ = [
    'env_config_merge',
    'env_config_evaluate',
    'env_pkg_header',
    'env_pkg_configure'
]

#---
# Internals
#---

def _env_config_patch(env, commit, env_key_bitmap):
    """ Patch missing key information

    @args
    > env            (dict) - { <KEY> : <value>, ... }
    > commit         (dict) - { <dep_name> : { <KEY> : <value>, ...}, ... }
    > env_key_bitmap (list) - already handled key information

    @return
    > Nothing
    """
    for dep_name in commit.keys():
        for dep_item in commit[dep_name].items():
            if dep_item[0] in env_key_bitmap:
                continue
            if dep_item[0] not in env:
                env[dep_item[0]] = dep_item[1]
                continue
            env[dep_item[0]] += dep_item[1]

def _env_config_preprocess(env, commit):
    """ Handle special '@<dep_name>@' information

    @args
    > env       (dict) - { <KEY> : <value>, ... }
    > commit    (dict) - { <dep_name> : { <KEY> : <value>, ...}, ... }

    @return
    > A list with all keys that has been handled
    """
    env_real = {}
    env_key_bitmap = []
    for env_key, env_data_list in env.items():
        # handle all preprocessing except for @COMMON@, that should be
        # performed later
        env_dep_bitmap = []
        env_data_almost = []
        for env_data in env_data_list:
            if env_data == '@COMMON@':
                env_data_almost.append(env_data)
                continue
            if env_data[0] != '@' or env_data[-1] != '@':
                env_data_almost.append(env_data)
                continue
            if not (env_data_target := env_data[1:-1]):
                log.warn("empty preprocessing indication")
                continue
            if env_data_target not in commit.keys():
                log.warn(f"unable to find env target '{env_data_target}'")
                continue
            if env_key not in commit[env_data_target]:
                continue
            env_data_almost.append(commit[env_data_target][env_key])
            env_dep_bitmap.append(env_data_target)

        # generate @COMMON@ data list with the rest
        env_data_common = []
        for dep_name in commit.keys():
            if dep_name in env_dep_bitmap or env_key not in commit[dep_name]:
                continue
            env_data_common += commit[dep_name][env_key]

        # replace all @COMMON@ indication
        env_data_real = []
        for env_data in env_data_almost:
            if env_data != '@COMMON@':
                env_data_real.append(env_data)
                continue
            if not env_data_common:
                log.warn('multiple @COMMON@ indication, skipped')
                continue
            env_data_real += env_data_common
            env_data_common = []

        # save the processed data
        env_real[env_key] = env_data_real

        # indicate which key as been handled. This information will be used to
        # "patch" missing key information
        env_key_bitmap.append(env_key)

    # return patched env
    _env_config_patch(env_real, commit, env_key_bitmap)
    return env_real

#---
# Public
#---

def env_config_merge(env, commit):
    """ Merge the current env configuration with a commit

    A commit is a collection of all dependencies "public" env configuration.
    Note that we should update the content of the `env` argument because this
    dictionary is linked to the DAG internal information.

    @args
    > env       (dict) - { <KEY> : <value>, ... }
    > commit    (dict) - { <dep_name> : { <KEY> : <value>, ...}, ... }

    @return
    > Nothing, `env` will be modified
    """
    log.debug(f"[env merge] env = {env} && commit = {commit}")
    env.update(_env_config_preprocess(env, commit))

def env_config_evaluate(env, glossary=None):
    """ Evaluate vars' content

    @args
    > env (dict) - { '<KEY>' : [ <value0>, <value1>, ...]

    @return
    > Nothing, `env` will be modified
    """
    eval_env = {}
    for key, data_list in env.items():
        eval_env[key] = []
        for data in data_list:
            eval_data = ''
            while True:
                if (sidx := data.find('{'))<0 or (eidx := data.find('}'))<0:
                    eval_data += data
                    break
                eval_data += data[:sidx]
                eval_key   = data[sidx + 1 : eidx]
                eval_key_data = env.get(eval_key)
                if glossary:
                    eval_key_data = glossary.get(eval_key)
                if eval_key_data:
                    eval_data += eval_key_data
                else:
                    log.warn(f" - '{key}' : unable to find")
                data = data[eidx + 1:]
            eval_env[key].append(eval_data)
    env.update(eval_env)

def env_pkg_header(pkg_meta):
    """ Default exported variable information

    =============================== =======================================
    Type                             Description
    =============================== =======================================
      VXSDK_PKG_NAME                project name
      VXSDK_PKG_TARGET              project target
      VXSDK_PKG_VERSION             project version
      VXSDK_PKG_IS_ORIGINAL         'true' if the package is the original
      VXSDK_PREFIX_BUILD            project build prefix (for object files)
      VXSDK_PREFIX_INSTALL          project installation prefix
      VXSDK_ASSETS_PREFIX_SRC       assets "build" directory
      VXSDK_BUILD_CFLAGS            include flags for GCC
      VXSDK_BUILD_LDFLAGS           include flags for GCC
      VXSDK_CURRENT_SOURCE_DIR      current package path
      VXSDK_PREFIX_SYSROOT          potential sysroot path for the package
    =============================== =======================================

    @arg
    > pkg_meta (obj) - pacakge meta information

    @return
    > Dictionary with default information
    """
    # alias
    prefix = pkg_meta.build_prefix

    # generate sysroot information
    pkg_prefix_sysroot = os.path.expanduser(config.get('path.sysroot'))
    pkg_prefix_sysroot = f"{pkg_prefix_sysroot}/{pkg_meta.name}"

    # generate VXSDK_PREFIX_BUILD information
    pkg_prefix_build = f"{prefix}/build/{pkg_meta.name}"
    if not os.path.exists(pkg_prefix_build):
        os.makedirs(pkg_prefix_build)

    # generate VXSDK_PREFIX_INSTALL information
    pkg_prefix_install = f"{prefix}/install"
    if pkg_meta.type == 'app':
        pkg_prefix_install = os.path.expanduser(config.get('path.bin'))
    if not os.path.exists(pkg_prefix_install):
        os.makedirs(pkg_prefix_install)

    # generate VXSDK_ASSETS_* information
    pkg_assets_src = f"{prefix}/converter"

    # generate VXSDK_PKG_* information
    pkg_name        = pkg_meta.name
    pkg_is_original = str(pkg_meta.is_original)

    # workaround to avoid CMake error with non-standar symver
    # (fixme/E8A7) : support version export
    pkg_version = pkg_meta.version
    pkg_version = '0.0.0'

    # generate "dependence-specific" env
    # (todo/97EA) : remove CFLAGS
    # (todo/97EA) : remove LDFLAGS
    return {
        'VXSDK_PKG_NAME'            : pkg_name,
        'VXSDK_PKG_TARGET'          : pkg_meta.target,
        'VXSDK_PKG_VERSION'         : pkg_version,
        'VXSDK_PKG_IS_ORIGINAL'     : pkg_is_original,
        'VXSDK_PREFIX_BUILD'        : pkg_prefix_build,
        'VXSDK_PREFIX_INSTALL'      : pkg_prefix_install,
        'VXSDK_ASSETS_PREFIX_SRC'   : pkg_assets_src,
        'VXSDK_BUILD_CFLAGS'        : '',
        'VXSDK_BUILD_LDFLAGS'       : '',
        'VXSDK_CURRENT_SOURCE_DIR'  : pkg_meta.path,
        'VXSDK_PREFIX_SYSROOT'      : pkg_prefix_sysroot
    }

def env_pkg_configure(pkg_meta, pkg_env, commit):
    """ Generate the complet package env configuration

    A commit is a collection of all dependencies "public" env configuration.
    Note that we should update the content of the `env` argument because this
    dictionary is linked to the DAG internal information.

    @args
    > pkg_meta (dict) - pacakge meta information
    > pkg_env  (dict) - pacakge env information
    > commit    (dict) - { <dep_name> : { <KEY> : <value>, ...}, ... }

    @return
    > Dictionary with all env variable setuped
    """
    # performs preproc pass
    log.debug(f"[{pkg_meta.name}] env_pkg_conf() -> commit = {commit}")
    env_config_merge(pkg_env, commit)

    # generate pacakge "header" env information
    # (todo/8DC1) : isolate the PKG ENV information (shouldn't be edited)
    envp = env_pkg_header(pkg_meta)

    # merge extra env configuration
    for item in pkg_env.items():
        data = item[1]
        if isinstance(item[1], (tuple, list)):
            data = ' '.join(item[1])
        if item[0] in envp:
            envp[item[0]] += f' {data}'
            continue
        envp[item[0]] = data

    # clean list for cmake
    for item in envp.items():
        data = item[1].split()
        envp[item[0]] = ';'.join(data)

    log.debug(f"[{pkg_meta.name}] generated env = {envp}")
    return envp
