"""
core.config - Configuration file wrapper
"""
import os
import toml

from core.logger import log

__all__ = [
    'config'
]

#---
# Internals
#---

__DEFAULT_CONFIG_KEYVAL = [
    ('path.config',   '~/.config/vxsdk/config.toml'),
    ('path.sysroot',  '~/.local/share/vxsdk/sysroot'),
    ('path.packages', '~/.local/share/vxsdk/packages'),
    ('path.assets',   '~/.local/share/vxsdk/assets'),
    ('path.bin',      '~/.local/bin'),

    ('build.default_target', 'superh'),

    ('pkg.backend.name', 'gitea'),
    ('pkg.backend.url',  'https://gitea.planet-casio.com'),
    ('pkg.local_storage', '{path.packages}')
]

class _VxConfigure():
    """ configuration abstraction class """
    def __init__(self, default_info):
        self._conf_file = {}
        self._conf_path = ''
        self._set_default(default_info)

    #---
    # Internals
    #---

    def _setitem_dots(self, dictionary, key, value, path=""):
        if "." not in key:
            old = dictionary[key] if key in dictionary else None
            dictionary[key] = value
            return old
        group, key = key.split(".", 1)
        if group in dictionary and not isinstance(dictionary[group], dict):
            raise ValueError(f"cannot assign {value} into value {path+group}")
        if group not in dictionary:
            dictionary[group] = {}
        return self._setitem_dots(
            dictionary[group], key, value, path + group + "."
        )


    def _config_control(self, name, value=None):
        """ Common configuration file manipulation

        If the `value` parameter is not set, only read operation will be
        performed, otherwise the complete file will be updated.

        @args
        > name  (str) - dot-separated ker (ex : `default.board.name`)
        > value (str) - value for the key
        """
        # check if config file information are cached
        # @notes
        # - create the configuration file folder if needed
        # - load the TOML content
        # - cache pathname to avoid path manipulation
        if not self._conf_path:
            self._conf_path = os.path.expanduser(
                '~/.config/vxsdk/config.toml'
            )
        if not self._conf_file:
            if os.path.exists(self._conf_path):
                with open(self._conf_path, 'r', encoding='utf8') as cfg:
                    self._conf_file = toml.loads(cfg.read())

        # check "read-only" request (just fetch value)
        if not value:
            conf = self._conf_file
            targets = name.split('.')
            while targets:
                if not targets[0] in conf:
                    log.debug(f"[config] unable to find target '{name}'")
                    return None
                conf = conf[targets[0]]
                targets = targets[1:]
            return conf

        # perform "write-only" request (update the configuration file)
        old = self._setitem_dots(self._conf_file, name, value)

        if not os.path.exists(self._conf_path):
            os.makedirs(os.path.dirname(self._conf_path))
        with open(self._conf_path, 'w+', encoding='utf-8') as file:
            file.write(toml.dumps(self._conf_file))

        # return the previous information of the update field (if available)
        return old


    def _generate_value(self, name, val):
        if not val:
            return None
        while val.find('{') >= 0:
            if val.find('}') < 0:
                break
            key = val[val.find('{') + 1: val.find('}')]
            res = self._config_control(key)
            if not res:
                log.warn(f"[config] {name} = {val} : unable to find '{key}'")
                return None
            val = val[:val.find('{')] + res + val[val.find('}') + 1:]
        return val

    def _set_default(self, list_of_keyval):
        """ Set default key / value

        This function will setup all default key value if the key doest not
        exists. This is usefull to centralise all default user configuration
        information in this file instead of in all project files.

        @arg
        > list_of_keyval: list of tuple - [(key, value), ...]
        """
        for key, value in list_of_keyval:
            if not self._config_control(key):
                self._config_control(key, value)

    #---
    # Public methods
    #---

    def get(self, key, default_value = None):
        """ Get configuration key/value

        This function will try to find the key value of `key`. If the key does
        not exists then None will be returned. You can specify a default value
        if the key doest not exist.

        @args
        > key: string           - the key name
        > default_value: string - the key default value if not found

        @return
        > return the key value or None if not found nor default value set
        """
        if ret := self._config_control(key):
            return self._generate_value(key, ret)
        default_value = self._generate_value(key, default_value)
        if default_value:
            self._config_control(key, default_value)
        return default_value

    def set(self, key, value):
        """ Set configuration key = value

        This function will try to update the user vxSDK configuration file to
        add key / value information. Note that the `value` can have placeholder
        in its content like `{path.sysroot}/superh` wich will fetch the
        'path.sysroot' configuration key.

        @args
        > key:  string - the key name
        > name: string - the key value

        @return
        > the old key value or None if new
        """
        return self._config_control(key, value)

#---
# Publics
#---

config = _VxConfigure(__DEFAULT_CONFIG_KEYVAL)
